/*
 * Copyright (c) 2011-2013 Alexandre Tuleu <alexandre.tuleu.2005@polytechnique.org>
 * Ecole Polytechnique Fédérale de Lausanne.
 *
 * this file is part of sbcpd.
 *
 * sbcpd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * sbcpd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with sbcpd.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * \file common.h
 *
 *  \date Aug 15, 2011
 *  \author Alexandre Tuleu
 */

#ifndef SBCPD_COMMON_H_
#define SBCPD_COMMON_H_

#ifndef NDEBUG
//#include <string.h>
//#define SBCPD_MAX_ERROR_STRING_SIZE 500
#endif

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>

/**
 * Typedef for byte.
 * \ingroup sbcpd_m
 */
typedef uint8_t sbcpd_byte_t;

/**
 * Type for the frame size.
 * \ingroup sbcpd_m
 */
typedef uint8_t sbcpd_frame_size_t;

/**
 * Type for the packet size.
 * \ingroup sbcpd_m
 */
typedef uint16_t sbcpd_packet_size_t;

/*******************************************************************************
 * FORWARD DECLARATION
 ******************************************************************************/
struct sbcpd_bus_settings_str;
typedef struct sbcpd_bus_settings_str sbcpd_bus_settings;

struct sbcpd_transfer_str;
typedef struct sbcpd_transfer_str sbcpd_transfer;

struct sbcpd_serial_interface_str;
typedef struct sbcpd_serial_interface_str sbcpd_serial_interface;


/**
 * \mainpage SBCP Low Level Drivers
 *
 * The Serial Byte Communication Protocol is a protocol developed by the AMARSi
 * project for the Oncilla Quadruped robot ( http://www.amarsi-project.eu).
 *
 * This protocol aims to make a SBCP Host to communicate to other devices using
 * a very lightweight serial protocol, which is optimized for low latency and
 * realtime applications.
 *
 * The protocol is implemented on Linux on two layer : the SBCP low level
 * drivers that manages all the communication logic, and the libsbcp that aims
 * to manages all the application logic of devices supporting the SBCP protocol.
 *
 */


#ifdef __cplusplus
}
#endif
#endif //SBCPD_COMMON_H_
