/*
 * Copyright (c) 2011-2013 Alexandre Tuleu <alexandre.tuleu.2005@polytechnique.org>
 * Ecole Polytechnique Fédérale de Lausanne.
 *
 * this file is part of sbcpd.
 *
 * sbcpd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * sbcpd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with sbcpd.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file tiny_set_flagger.h
 *
 *  \date Apr 11, 2012
 *  \author Alexandre Tuleu
 */

#ifndef SBCPD_TINY_SET_MANAGER_H_
#define SBCPD_TINY_SET_MANAGER_H_

#include <stdint.h>
#include <string.h>

#include <sbcpd/transfer.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct sbcpd_dl_item_str sbcpd_dl_item;

struct sbcpd_dl_item_str {
	sbcpd_transfer  * t;
	sbcpd_dl_item   * prev;
	sbcpd_dl_item   * next;
};

struct sbcpd_dl_list_str {
	sbcpd_dl_item * begin;
	sbcpd_dl_item * end;
};
typedef struct sbcpd_dl_list_str sbcpd_dl_list;

void sbcpd_init_dl(sbcpd_dl_list * l);

void sbcpd_free_dl(sbcpd_dl_list * l);

void sbcpd_dl_push_back(sbcpd_dl_list * l, sbcpd_dl_item * i);

sbcpd_dl_item * sbcpd_dl_pop_front(sbcpd_dl_list * l);


int sbcpd_dl_list_empty(sbcpd_dl_list * l);


void sbcpd_dl_remove_item_from_list(sbcpd_dl_list * l,sbcpd_dl_item * i);
/**
 * Tags element of a set defined by an id < 256 in O(1).
 */
struct sbcpd_transfer_manager_str {
	uint8_t size;

	sbcpd_transfer * transfers;

	sbcpd_dl_list * available_transfers;

	sbcpd_dl_list * allocated_transfers;

	sbcpd_dl_item ** allocated_by_id;

	uint8_t allocated_size;

};
typedef struct sbcpd_transfer_manager_str sbcpd_transfer_manager;


/**
 * Inits the manager;
 * @param tsf
 */
void sbcpd_tm_init(sbcpd_transfer_manager * tm,uint8_t size);

void sbcpd_tm_free(sbcpd_transfer_manager * tm);

sbcpd_transfer * sbcpd_tm_allocate_transfer(sbcpd_transfer_manager * tm);

int sbcpd_tm_is_allocated(sbcpd_transfer_manager * tm, sbcpd_transfer * t);

void sbcpd_tm_release_transfer(sbcpd_transfer_manager * tm, sbcpd_transfer * t);


sbcpd_transfer * sbcpd_tm_first_allocated_transfer(sbcpd_transfer_manager * tm);

sbcpd_transfer * sbcpd_tm_next_allocated_transfer(sbcpd_transfer_manager * tm,
                                                    sbcpd_transfer * t);


uint8_t sbcpd_tm_transfer_count(sbcpd_transfer_manager * tm);

#ifdef __cplusplus
}
#endif
#endif // SBCPD_TINY_SET_FLAGGER_H_
