/*
 * Copyright (c) 2011-2013 Alexandre Tuleu <alexandre.tuleu.2005@polytechnique.org>
 * Ecole Polytechnique Fédérale de Lausanne.
 *
 * this file is part of sbcpd.
 *
 * sbcpd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * sbcpd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with sbcpd.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * \file sbcp.h
 *
 * \date Oct 11, 2011
 * \author Alexandre Tuleu
 */
#ifndef SBCPD_SBCP_H_
#define SBCPD_SBCP_H_

#ifdef __cplusplus
extern "C" {
#endif

enum sbcpd_sbcp_index_enm {
  SBCP_HEADER_POS = 0   ,
  SBCP_CLASS_POS        ,
  SBCP_ID_POS           ,
  SBCP_PAYLOAD_SIZE_POS ,
  SBCP_INSTRUCTION_POS  ,
  SBCP_PAYLOAD_START_POS
};


enum sbcpd_sbcp_error_enm {
	SBCP_NO_ERROR                = 0x00,
	SBCP_APPLICATION_ERROR_MASK  = 0x7f,
	SBCP_COMMUNICATION_ERROR_BIT = 0x80
};

enum sbcpd_sbcp_communication_error_enm {
    SBCP_CERR_CS_INCORRECT        = 0x80,
    SBCP_CERR_BUS_TIMEOUT         = 0x81,
    SBCP_CERR_FRAME_ERROR         = 0x82,
    SBCP_CERR_UNKNOWN_INSTRUCTION = 0x83,
    SBCP_CERR_BUS_FRAME_ERROR     = 0x84,
    SBCP_CERR_HOST_TIMEOUT        = 0x85,
    SBCP_CERR_HOST_FRAME_ERROR    = 0x86,
    SBCP_CERR_SLAVE_TIMEOUT       = 0x87,
    SBCP_CERR_SLAVE_FRAME_ERROR   = 0x88,
    SBCP_CERR_UNEXPECTED_SIZE     = 0x89,
    SBCP_CERR_REG_NOT_READABLE    = 0x8a,
    SBCP_CERR_REG_NOT_WRITABLE    = 0x8b,
    SBCP_CERR_REG_PROTECTED       = 0x8c,
    SBCP_CERR_UNIMPLEMENTED_INSTRUCTION = 0x8d
};


static const unsigned int SBCP_MINIMAL_SIZE = 6;

static const uint8_t SBCP_HEADER_VALUE = 0xff;

static const uint8_t SBCP_MASTER_CLASS = 0x00;

static const uint8_t SBCP_MASTER_BROADCAST_ID = 0xff;

static const uint8_t SBCP_GET_FIRMWARE_INSTRUCTION = 0x0;

static const uint8_t SBCP_GET_REGISTER_INSTRUCTION = 0x01;

static const uint8_t SBCP_MASTER_FRAME_SIZE_REG_ADDRESS = 0x08;

static const uint8_t SBCP_MASTER_PACKET_SIZE_REG_ADDRESS = 0x09;


#ifdef __cplusplus
}
#endif

#endif //SBCPD_SBCP_H_
