/**
 * \file settings.c
 *
 *  \date Apr 12, 2012
 *  \author Alexandre Tuleu
 */

#include "settings.h"

#include <stdlib.h>
#include<stdio.h>
#include "error.h"

sbcpd_bus_settings * sbcpd_bus_settings_create(){
	sbcpd_bus_settings * s = malloc(sizeof(sbcpd_bus_settings));
	if(s == NULL){
		return s;
	}

	s->frame_rate = 5;
	s->max_trials = 20;
	s->minimal_frame_size = 4;
	s->minimal_packet_size = 32;
	s->monitor = 0;
	return s;
}


void sbcpd_bus_settings_free(sbcpd_bus_settings * settings){
	if(settings){
		free(settings);
	}
}


int sbcpd_bus_settings_set_master_id(sbcpd_bus_settings * settings,
                                      uint8_t master_id){
	fprintf(stderr,"sbcpd_bus_settings_set_master_id() is depreciated. Master ID is now autonatically found. This call has no effect.");
	return SBCPD_NO_ERROR;
}

int sbcpd_bus_settings_set_nb_trials_per_packet(sbcpd_bus_settings * settings,
                                                uint8_t nb_trials){
	if(settings){
		settings->max_trials = nb_trials;
	} else {
		return SBCPD_ESET_NOT_ALLOCATED;
	}

	return SBCPD_NO_ERROR;
}

int sbcpd_bus_settings_set_required_frame_definition(sbcpd_bus_settings * settings,
                                                     sbcpd_frame_size_t packet_per_frame,
                                                     sbcpd_packet_size_t packet_size){
	if(settings){
		settings->minimal_frame_size = packet_per_frame;
		settings->minimal_packet_size = packet_size;
	} else {
		return SBCPD_ESET_NOT_ALLOCATED;
	}

	return SBCPD_NO_ERROR;
}

int sbcpd_bus_settings_set_monitoring(sbcpd_bus_settings * settings,
                                      uint8_t enable){
	if (settings == 0){
		return SBCPD_ESET_NOT_ALLOCATED;
	}
	settings->monitor = (enable == 0 ) ? 0 : 1;
	return SBCPD_NO_ERROR;
}
