/**
 * \file settings.h
 *
 *  \date Apr 12, 2012
 *  \author Alexandre Tuleu
 */

#ifndef SBCPD_SETTINGS_H_
#define SBCPD_SETTINGS_H_

#include <stdint.h>

#include "common.h"

#ifdef __cplusplus
extern "C" {
#endif

enum sbcpd_bus_settings_error {
	SBCPD_ESET_NOT_ALLOCATED,
	SBCPD_ESET_DEPRECIATED
};



 /**
  *  Settings of the bus that should be set by the user before opening it.
  *  \ingroup sbcpd_bus_m
  */
struct sbcpd_bus_settings_str  {
	unsigned int frame_rate;

	unsigned int max_trials;

	sbcpd_frame_size_t  minimal_frame_size;
	sbcpd_packet_size_t minimal_packet_size;

	uint8_t             monitor;

};

/**
 * Creates a new setting
 * @return
 */
sbcpd_bus_settings * sbcpd_bus_settings_create();

/**
 * free a setting
 * @param settings
 */
void sbcpd_bus_settings_free(sbcpd_bus_settings * settings);

/**
 * Sets the ID of the master board for the network.
 * @param settings
 * @param master_id
 */
int sbcpd_bus_settings_set_master_id(sbcpd_bus_settings * settings,
                                      uint8_t master_id);

/**
 * Sets the maximal number of time a packet will try to be read before abording.
 * @param settings
 * @param nb_trials
 * @return
 */
int sbcpd_bus_settings_set_nb_trials_per_packet(sbcpd_bus_settings  * settings,
                                                uint8_t nb_trials);

/**
 * Sets the required frame definition for the user.
 * @param packet_per_frame nb consecutive packet in one frame.
 * @param packet_size size of a single packet
 * @return
 */
int sbcpd_bus_settings_set_required_frame_definition(sbcpd_bus_settings  * settings,
                                                     sbcpd_frame_size_t packet_per_frame,
                                                     sbcpd_packet_size_t packet_size);


/**
 * Enables or disable the monitoring of bus error.
 */
int sbcpd_bus_settings_set_monitoring(sbcpd_bus_settings * settings,
                                      uint8_t enable);

#ifdef __cplusplus
}
#endif
#endif // SBCPD_SETTINGS_H_
