/*
 * Copyright (c) 2011-2013 Alexandre Tuleu <alexandre.tuleu.2005@polytechnique.org>
 * Ecole Polytechnique Fédérale de Lausanne.
 *
 * this file is part of sbcpd.
 *
 * sbcpd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * sbcpd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with sbcpd.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * \file ft2232h.c
 *
 *  \date Oct 10, 2011
 *  \author Alexandre Tuleu
 */


#include "ft2232h.h"

#include <sbcpd/ftdi/ft2232h.h>
#include <ftdi.h>


sbcpd_serial_interface * sbcpd_ft2232h_create_interface(){
	sbcpd_serial_interface * si = malloc(sizeof(sbcpd_serial_interface));

	struct ftdi_context * ftdi =  malloc(sizeof(struct ftdi_context));

	int f;

	if((f = ftdi_init(ftdi)) < 0){
		//report it
		return si;
	}

	si->dev = ftdi;
	si->read_data  = &sbcpd_ft2232h_read;
	si->write_data = &sbcpd_ft2232h_write;
	si->close = &sbcpd_ft2232h_close;
	si->free  = &sbcpd_ft2232h_free;
	return si;

}


int sbcpd_ft2232h_open(sbcpd_serial_interface * si,
                       unsigned int baudrate,
                       unsigned char latency,
                       unsigned int interface){
	ftdi_set_interface(si->dev, interface < 6? interface : 5 );

	int f;
	if((f = ftdi_usb_open(si->dev,0x403,0x6010)) < 0 ) {
		si->error.code = (uint32_t)SBCPD_ESI_CANNOT_ACCESS_DEVICE;
		si->error.msg = ftdi_get_error_string(si->dev);
		return SBCPD_ESI_CANNOT_ACCESS_DEVICE;
	}

	if ((f = ft2232h_set_baudrate(si->dev,(int)baudrate)) < 0 ){
		si->error.code = (uint32_t)SBCPD_ESI_CONFIGURE_ERROR;
		si->error.msg = ftdi_get_error_string(si->dev);
		return SBCPD_ESI_CONFIGURE_ERROR;
	}

	if ((f = ft2232h_set_latency_timer(si->dev,latency)) < 0){
		si->error.code = (uint32_t)SBCPD_ESI_CONFIGURE_ERROR;
		si->error.msg = ftdi_get_error_string(si->dev);
		return SBCPD_ESI_CONFIGURE_ERROR;
	}

	if((f = ftdi_usb_purge_buffers(si->dev)) < 0){
		si->error.code = (uint32_t)SBCPD_ESI_CONFIGURE_ERROR;
		si->error.msg = ftdi_get_error_string(si->dev);
		return SBCPD_ESI_CONFIGURE_ERROR;
	}
	return SBCPD_ESI_NO_ERROR;
}

void sbcpd_ft2232h_close(sbcpd_serial_interface *si){
	ftdi_deinit(si->dev);
}

void sbcpd_ft2232h_free(sbcpd_serial_interface *si){
	sbcpd_ft2232h_close(si);
	ftdi_free(si->dev);
}

int sbcpd_ft2232h_write(sbcpd_serial_interface * si,
                        const sbcpd_byte_t * data,
                        int size){
	int ret = ftdi_write_data(si->dev,(uint8_t*)data,size);
	if(ret < 0){
		si->error.code = (uint32_t)SBCPD_ESI_WRITE_ERROR;
		si->error.msg  = ftdi_get_error_string(si->dev);
		return SBCPD_ESI_WRITE_ERROR;
	}
	return ret;
}

int sbcpd_ft2232h_read(sbcpd_serial_interface * si,
                       sbcpd_byte_t * data,
                       int size){
	int ret = ftdi_read_data(si->dev,data,size);
	if(ret < 0){
		si->error.code = (uint32_t)SBCPD_ESI_READ_ERROR;
		si->error.msg  = ftdi_get_error_string(si->dev);
		return SBCPD_ESI_READ_ERROR;
	}
	return ret;
}


