/*
 * Copyright (c) 2011-2013 Alexandre Tuleu <alexandre.tuleu.2005@polytechnique.org>
 * Ecole Polytechnique Fédérale de Lausanne.
 *
 * this file is part of sbcpd.
 *
 * sbcpd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * sbcpd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with sbcpd.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * \file serial_interface.h
 *
 *  \date Oct 6, 2011
 *  \author Alexandre Tuleu
 */

#ifndef SBCPD_SI_SERIAL_INTERFACE_H_
#define SBCPD_SI_SERIAL_INTERFACE_H_

#include <stdint.h>
#include <sbcpd/common.h>
#include <sbcpd/error.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Errors that can be returned by the modules functions.
 */
enum sbcpd_serial_interface_error_enm {
	SBCPD_ESI_NO_ERROR             =  0,//!< SBCPD_ESI_NO_ERROR
	SBCPD_ESI_CANNOT_ACCESS_DEVICE = -1,//!< SBCPD_ESI_CANNOT_ACCESS_DEVICE
	SBCPD_ESI_CONFIGURE_ERROR      = -2,//!< SBCPD_ESI_CONFIGURE_ERROR
	SBCPD_ESI_READ_ERROR           = -3,//!< SBCPD_ESI_READ_ERROR
	SBCPD_ESI_WRITE_ERROR          = -4 //!< SBCPD_ESI_WRITE_ERROR
};


typedef int (*sbcpd_si_write) (sbcpd_serial_interface * dev,
                               const sbcpd_byte_t * data,
                               int size);

typedef int (*sbcpd_si_read)  (sbcpd_serial_interface * dev,
                               sbcpd_byte_t * data,
                               int size);


typedef void (*sbcpd_si_close)(sbcpd_serial_interface * si);
typedef void (*sbcpd_si_free)(sbcpd_serial_interface * si);

/**
 * Represents a serial interface used to do Serial I/O from user space.
 */
struct sbcpd_serial_interface_str {
	sbcpd_si_write        write_data;
	sbcpd_si_read         read_data;
	sbcpd_si_close        close;
	sbcpd_si_free         free;
	void * dev;
	sbcpd_error           error;
};

#ifdef __cplusplus
}
#endif
#endif // SBCPD_SI_SERIAL_INTERFACE_H_
