/*
 * Copyright (c) 2013 Alexandre Tuleu <alexandre.tuleu.2005@polytechnique.org>
 * Ecole Polytechnique Fédérale de Lausanne.
 *
 * this file is part of sbcpd.
 *
 * sbcpd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * sbcpd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with sbcpd.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * \file monitor.h
 *
 *  \date May 20, 2013
 *  \author Alexandre Tuleu
 */

#ifndef SBCPD_STATS_H_
#define SBCPD_STATS_H_

#include <stdint.h>
#include "common.h"
#include "transfer.h"

#ifdef __cplusplus
extern "C" {
#endif //__cplusplus

/**
 * The type representing a number of transaction over a bus. Considering 10
 * packet per ms (this is not possible yet), we will overloww in more than
 * 6000 years.
 */
typedef uint64_t sbcpd_nb_packet_t;

struct sbcpd_stats_str;
typedef struct sbcpd_stats_str sbcpd_stats;


/**
 * Returns the number of sbcpd packet transmitted on the bus over time.
 */
sbcpd_nb_packet_t sbcpd_stats_get_nb_packet(sbcpd_stats * s);

/**
 * Returns the number of packets that ended with a slave application errors
 */
sbcpd_nb_packet_t sbcpd_stats_get_slave_application_errors(sbcpd_stats * s);

/**
 * Returns the number of packet that ended with a communication error
 * \param type : the communication error to test for. if SBCPD_CERR_NO_ERROR is
 *               passed, all errors are considered.
 */
sbcpd_nb_packet_t sbcpd_stats_get_nb_com_errors(sbcpd_stats * s,
                                                sbcpd_transfer_communication_error type);

/**
 * Prints to file a short human readable version
 */
void sbcpd_stats_fprintf(FILE * f, sbcpd_stats * s);

/**
 * Prints to file a more complete description.
 */
void sbcpd_stats_detailed_fprintf(FILE * f, sbcpd_stats * s);


#ifdef __cplusplus
}
#endif //__cplusplus


#endif /* SBCPD_MONITOR_H_ */


