/*
 * Copyright (c) 2011-2013 Alexandre Tuleu <alexandre.tuleu.2005@polytechnique.org>
 * Ecole Polytechnique Fédérale de Lausanne.
 *
 * this file is part of sbcpd.
 *
 * sbcpd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * sbcpd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with sbcpd.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file DoubleLinkedListUTest.cpp
 *
 *  \date Apr 13, 2012
 *  \author Alexandre Tuleu
 */

#include "DoubleLinkedListUTest.h"


void DoubleLinkedListUTest::SetUp(){
	list = new sbcpd_dl_list();
	sbcpd_init_dl(list);
	for(unsigned int i = 0; i < 4 ; ++i){
		it[i] = (sbcpd_dl_item*) malloc(sizeof(sbcpd_dl_item));
		sbcpd_dl_push_back(list,it[i]);
	}
}

void DoubleLinkedListUTest::TearDown(){
	sbcpd_free_dl(list);
	delete list;
	list = NULL;
}

TEST_F(DoubleLinkedListUTest,NewlyCreatedListIsEmpty){
	sbcpd_dl_list * foo = new sbcpd_dl_list();
	EXPECT_EQ(NULL,foo->begin);
	EXPECT_EQ(NULL,foo->end);

	delete foo;

}

void DoubleLinkedListUTest::TestListIntegrity(unsigned int nbEl){
	sbcpd_dl_item * its = list->begin;
	unsigned int nbElements(0);


	while(its){
		if(its != list->begin){
			ASSERT_NE((sbcpd_dl_item*)NULL,its->prev);
			EXPECT_EQ(its,its->prev->next);
		} else {
			EXPECT_EQ(NULL,its->prev);
		}

		if(its !=list->end){
			ASSERT_NE((sbcpd_dl_item*)NULL,its->next);
			EXPECT_EQ(its,its->next->prev);
		} else {
			EXPECT_EQ(NULL,its->next);
		}
		its = its->next;
		++nbElements;
	}

	EXPECT_EQ(nbEl,nbElements);

}

TEST_F(DoubleLinkedListUTest,ElementLinkingIsOk){
	TestListIntegrity();
}


TEST_F(DoubleLinkedListUTest,PushingPreserveIntegrity){
	sbcpd_dl_item * foo = (sbcpd_dl_item *) malloc(sizeof(sbcpd_dl_item ));
	sbcpd_dl_push_back(list,foo);
	TestListIntegrity(5);
}

TEST_F(DoubleLinkedListUTest,PopingPreserveIntegrity){
	sbcpd_dl_item * first;
	ASSERT_NO_FATAL_FAILURE(( first = sbcpd_dl_pop_front(list)));

	ASSERT_NO_FATAL_FAILURE(TestListIntegrity(3));

	ASSERT_NE((sbcpd_dl_item*)NULL,first);
	EXPECT_EQ(NULL,first->prev);
	EXPECT_EQ(NULL,first->next);

	EXPECT_NE((sbcpd_dl_item*)NULL,list->begin);

	ASSERT_NO_FATAL_FAILURE(free(first));
}

TEST_F(DoubleLinkedListUTest,ElementOrderIsPreserved){
	sbcpd_dl_item * item = list->begin;
	unsigned int index(0);

	while(item){
		EXPECT_EQ(it[index],item);

		item = item->next;
		++index;
	}
}

TEST_F(DoubleLinkedListUTest,IntegrityPreservedOnRemoval){
	EXPECT_NO_FATAL_FAILURE(sbcpd_dl_remove_item_from_list(list,it[2]));
	TestListIntegrity(3);

	EXPECT_NO_FATAL_FAILURE(sbcpd_dl_remove_item_from_list(list,it[0]));
	TestListIntegrity(2);

	EXPECT_NO_FATAL_FAILURE(sbcpd_dl_remove_item_from_list(list,it[3]));
	TestListIntegrity(1);
}

TEST_F(DoubleLinkedListUTest,EmptyingListIsWorking){
	EXPECT_NO_FATAL_FAILURE(free(sbcpd_dl_pop_front(list)));
	EXPECT_NO_FATAL_FAILURE(free(sbcpd_dl_pop_front(list)));
	EXPECT_NO_FATAL_FAILURE(free(sbcpd_dl_pop_front(list)));
	EXPECT_NO_FATAL_FAILURE(free(sbcpd_dl_pop_front(list)));
	TestListIntegrity(0);

	EXPECT_EQ((sbcpd_dl_item*)NULL,list->begin);
	EXPECT_EQ((sbcpd_dl_item*)NULL,list->end);
}

