/*
 * Copyright (c) 2011-2013 Alexandre Tuleu <alexandre.tuleu.2005@polytechnique.org>
 * Ecole Polytechnique Fédérale de Lausanne.
 *
 * this file is part of sbcpd.
 *
 * sbcpd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * sbcpd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with sbcpd.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * \file FakeSerialInterface.cpp
 *
 *  \date Apr 16, 2012
 *  \author Alexandre Tuleu
 */

#include "FakeSerialInterface.h"

#include <sbcpd/sbcp.h>

#include <cstring>

#include <stdexcept>
FakeSerialInterface::FakeSerialInterface(){
	pthread_mutex_init(&mutex,NULL);
	si = new sbcpd_serial_interface();
	si->dev = this;
	si->write_data = & fake_si_write;
	si->read_data =  & fake_si_read;
	si->close = &fake_si_close;
	si->free = & fake_si_free;
}

FakeSerialInterface::~FakeSerialInterface(){
}


sbcpd_serial_interface * FakeSerialInterface::SI(){
	return si;
}

sbcpd_byte_t * FakeSerialInterface::AppendResponse(sbcpd_packet_size_t size){
	Lock();
	responses.push_back(Packet());
	responses.back().in_size = size;
	sbcpd_byte_t * res =responses.back().in;
	UnLock();
	return res;
}
extern "C" {
	int fake_si_write (sbcpd_serial_interface * si,
	                   const sbcpd_byte_t * data,
	                   int size){
		FakeSerialInterface * dev = static_cast<FakeSerialInterface*>(si->dev);
		dev->Lock();
		int res = dev->Write(data,size);
		dev->UnLock();
		return res;
	}


	int fake_si_read(sbcpd_serial_interface * si,
	                 sbcpd_byte_t * data,
	                 int size){
		FakeSerialInterface *dev = static_cast<FakeSerialInterface*>(si->dev);
		dev->Lock();
		if(dev->responses.empty()){

			int res = dev->Read(data,size);
			dev->UnLock();
			return res;
		}
		unsigned int actualSize = std::min((sbcpd_packet_size_t)size,
                                           dev->responses.front().in_size);
		for(unsigned int i = 0; i < actualSize; ++i){
			data[i] = dev->responses.front().in[i];
		}
		dev->responses.pop_front();

		int res = dev->Read(data,size);
		dev->UnLock();
		return res;
	}


	void fake_si_close(sbcpd_serial_interface * si){
		static_cast<FakeSerialInterface*>(si->dev)->Close();
	}


	void fake_si_free(sbcpd_serial_interface * si){
		static_cast<FakeSerialInterface*>(si->dev)->Free();
	}
}

void FakeSerialInterface::Lock(){
	pthread_mutex_lock(&mutex);
}


void FakeSerialInterface::UnLock(){
	pthread_mutex_unlock(&mutex);
}

FakeSerialInterface::Packet::Packet()
	: in(new sbcpd_byte_t[261]){
}

FakeSerialInterface::Packet::~Packet(){
	delete [] in;
}


FakeSerialInterface::Packet::Packet(const Packet & src)
	: in(new sbcpd_byte_t[261])
	, in_size(src.in_size){
	memcpy(in,src.in,261*sizeof(sbcpd_byte_t));
}

