/*
 * Copyright (c) 2011-2013 Alexandre Tuleu <alexandre.tuleu.2005@polytechnique.org>
 * Ecole Polytechnique Fédérale de Lausanne.
 *
 * this file is part of sbcpd.
 *
 * sbcpd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * sbcpd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with sbcpd.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * \file FakeSerialInterface.h
 *
 *  \date Apr 16, 2012
 *  \author Alexandre Tuleu
 */

#ifndef SBCPD_FAKESERIALINTERFACE_H_
#define SBCPD_FAKESERIALINTERFACE_H_

#include <gmock/gmock.h>

#include <sbcpd/si/serial_interface.h>

#include <list>
#include <pthread.h>

extern "C" {
	int fake_si_write (sbcpd_serial_interface * dev,
	                   const sbcpd_byte_t * data,
	                   int size);

	int fake_si_read(sbcpd_serial_interface * dev,
	                 sbcpd_byte_t * data,
	                 int size);


	void fake_si_close(sbcpd_serial_interface * si);
	void fake_si_free(sbcpd_serial_interface * si);
}

/**
 *
 */
class FakeSerialInterface {
public :
	FakeSerialInterface();
	virtual ~FakeSerialInterface();

	sbcpd_serial_interface * SI();

	virtual int Write(const sbcpd_byte_t * data,
	                  int size) = 0;

	virtual int Read(sbcpd_byte_t * data,
	                 int size) = 0;

	virtual void Close() = 0;

	virtual void Free() = 0;

	sbcpd_byte_t * AppendResponse(sbcpd_packet_size_t size);

	void Lock();
	void UnLock();

private :

	friend int fake_si_read(sbcpd_serial_interface * dev,
	                        sbcpd_byte_t * data,
	                        int size);

	sbcpd_serial_interface * si;

//	sbcpd_byte_t out[261];
//	sbcpd_packet_size_t out_size;
	struct Packet {
		sbcpd_byte_t * in;
		sbcpd_packet_size_t in_size;
		Packet();
		~Packet();
		Packet(const Packet & src);
	};

	typedef std::list<Packet> StackOfResponse;

	StackOfResponse responses;
	pthread_mutex_t mutex;
};


class MockSerialInterface : public FakeSerialInterface {
public :
	MOCK_METHOD2(Write,int(const sbcpd_byte_t * data,int size));
	MOCK_METHOD2(Read,int(sbcpd_byte_t * data,int size));
	MOCK_METHOD0(Close,void());
	MOCK_METHOD0(Free,void());
};
#endif // SBCPD_FAKESERIALINTERFACE_H_
