/*
 * Copyright (c) 2011-2013 Alexandre Tuleu <alexandre.tuleu.2005@polytechnique.org>
 * Ecole Polytechnique Fédérale de Lausanne.
 *
 * this file is part of sbcpd.
 *
 * sbcpd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * sbcpd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with sbcpd.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file transfer.c
 *
 *  \date Aug 15, 2011
 *  \author Alexandre Tuleu
 */

#include "transfer.h"

const char * sbcpd_get_transfer_status_str(sbcpd_transfer_status s){
	switch (s){
	case SBCPD_TRANSFER_OK :
		return "Ok";
	case SBCPD_TRANSFER_UNSTARTED :
		return "Unstarted";
	case SBCPD_TRANSFER_IN_PROGRESS :
		return "In progress";
	case SBCPD_TRANSFER_ABORTED :
		return "Aborted";
	case SBCPD_TRANSFER_HAS_COMMUNICATION_ERROR :
		return "Has communication error";
	case SBCPD_TRANSFER_HAS_APPLICATION_ERROR :
		return "HAs application error";
	default :
		return "Unknown state";
	}
}

const char * sbcpd_get_transfer_com_error_str(sbcpd_transfer_communication_error com_error){
	switch(com_error){
	case SBCPD_CERR_NO_ERROR :
		return "No error";
	case SBCPD_CERR_BAD_CERR_PACKET :
		return "Bad communication error packet";
	case SBCPD_CERR_IMCOMPLETE_READOUT :
		return "Not enough data where received to read the response packet";
	case SBCPD_CERR_INCONSISTENT_RESPONSE_SIZE :
		return "The size of the response is not what was expected";
	case SBCPD_CERR_HOST_FRAME_ERROR :
		return "Master of the bus got an UART frame error from the host";
	case SBCPD_CERR_MASTER_FRAME_ERROR :
		return "The targeted slave got an UART frame error from the master";
	case SBCPD_CERR_SLAVE_FRAME_ERROR :
		return "The master got an UART frame error from the targeted slave";
	case SBCPD_CERR_HOST_TIMEDOUT :
		return "The master issued a timeout while receiving from the host";
	case SBCPD_CERR_MASTER_TIMEDOUT :
		return "The slave issued a timeout while receiving from the slave";
	case SBCPD_CERR_SLAVE_TIMEOUT :
		return "The master issued a timeout while receiving from the slave";
	case SBCPD_CERR_HOST_BAD_CHECKSUM :
		return "The master issued a bad checksum from the host packet";
	case SBCPD_CERR_MASTER_BAD_CHECKSUM :
		return "The slave issued a bad checksum from the master packet";
	case SBCPD_CERR_SLAVE_BAD_CHECKSUM :
		return "The master issued a bad checksum from the slave packet";
	case SBCPD_CERR_UNKNOWN_INSTRUCTION_ERROR :
		return "Unknown error code in the packet";
	default :
		return "Unknown";
	}
}

uint8_t sbcpd_transfer_get_uuid(sbcpd_transfer * t){
	return t->id;
}

void sbcpd_transfer_fprintf(sbcpd_transfer * t, FILE * f){
	fprintf(f,
	        "Transfef UUID : %d \nstatus : %s\ncommunication error : %s\nhas a callback : %s\n",
	        t->id,
	        sbcpd_get_transfer_status_str(t->status),
	        sbcpd_get_transfer_com_error_str(t->com_error),
	        (t->callback == NULL ) ? "no" : "yes");
}

const char * sbcp_transfer_status_msg(sbcpd_transfer_status s){
	return sbcpd_get_transfer_status_str(s);
}


